package com.jz.agent.client.service;

import com.google.gson.Gson;

import com.jz.agent.client.DmpDsAgentClient;
import com.jz.agent.utils.JDBCUtils;
import com.jz.dmp.agent.DmpAgentResult;
import com.jz.dmp.agent.ResultCode;
import com.jz.dmp.modules.model.DmpAgentDatasourceInfo;
import org.apache.hadoop.conf.Configuration;
import org.apache.hadoop.security.UserGroupInformation;
import org.apache.kudu.ColumnSchema;
import org.apache.kudu.Type;
import org.apache.kudu.client.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public class DmpKuduAgentClientService implements DmpDsAgentClient {

    private Gson gson = new Gson();

    private static Logger logger = LoggerFactory.getLogger(DmpKuduAgentClientService.class);

    public DmpAgentResult testConnect(DmpAgentDatasourceInfo ds) {
        Connection connection = null;
        String jdbcUrl = ds.getJdbcUrl();
        try {
            jdbcUrl += securityConnect(ds);

            Class.forName(ds.getDriverClassName());
            connection = DriverManager.getConnection(jdbcUrl);
            connection.close();
            return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(true));
        } catch (Exception e) {
            logger.error("KUDU连通性异常", e);
            return new DmpAgentResult(ResultCode.EXCEPTION, "KUDU连通性异常：" + e.getMessage());
        } finally {
            JDBCUtils.disconnect(connection, null, null);
        }
    }

    public DmpAgentResult getTableNameList(DmpAgentDatasourceInfo ds) {
        List<String> list = new ArrayList();
        Connection connection = null;
        PreparedStatement ps = null;
        ResultSet rs = null;
        String jdbcUrl = ds.getJdbcUrl();
        String sql = "show tables";
        try {
            jdbcUrl += securityConnect(ds);

            Class.forName(ds.getDriverClassName());
            connection = DriverManager.getConnection(jdbcUrl);
            ps = connection.prepareStatement(sql);
            rs = ps.executeQuery();
            while (rs.next()) {
                list.add(rs.getString("name"));
            }
        } catch (Exception e) {
            logger.error("KUDU获取表列表异常", e);
            return new DmpAgentResult(ResultCode.EXCEPTION, "KUDU获取表列表异常：" + e.getMessage());
        } finally {
            JDBCUtils.disconnect(connection, rs, ps);
        }
        return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(list));
    }

    public DmpAgentResult getTableColumnList(DmpAgentDatasourceInfo ds, String tableName) {
        List<Map<String, Object>> list = new ArrayList();
        Connection connection = null;
        PreparedStatement ps = null;
        ResultSet rs = null;
        String jdbcUrl = ds.getJdbcUrl();
        String sql = "DESCRIBE " + tableName;
        try {
            jdbcUrl += securityConnect(ds);

            Class.forName(ds.getDriverClassName());
            connection = DriverManager.getConnection(jdbcUrl);
            ps = connection.prepareStatement(sql);
            rs = ps.executeQuery();
            while (rs.next()) {
                Map<String, Object> m = new HashMap<String, Object>();
                m.put("name", rs.getString("name").toLowerCase());
                m.put("type", convertColumnType(rs.getString("type").toUpperCase()));
                m.put("comment", rs.getString("comment"));
                list.add(m);
            }
        } catch (Exception e) {
            logger.error("KUDU获取表属性异常", e);
            return new DmpAgentResult(ResultCode.EXCEPTION, "KUDU获取表属性异常：" + e.getMessage());
        } finally {
            JDBCUtils.disconnect(connection, rs, ps);
        }
        return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(list));
    }

    public DmpAgentResult previewData(DmpAgentDatasourceInfo ds, String tableName) {
        Map<String, Object> map = new HashMap();
        return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(map));
    }

    /**
     * 安全连接并返回jdbc连接url的安全请求后缀
     *
     * @param ds
     * @return jdbc连接url的安全请求后缀
     */
    private String securityConnect(DmpAgentDatasourceInfo ds) {
        String jdbcUrlSuffix = "";
        if ("true".equalsIgnoreCase(ds.getKerberosIsenable())) {
            // 不从系统配置中取kerberosFqdn
            jdbcUrlSuffix = ";AuthMech=1;KrbHostFQDN=" + ds.getAccessId() + ";KrbServiceName=impala";
            System.setProperty("java.security.krb5.conf", ds.getKerberosKrb5Conf());
            System.setProperty("java.security.auth.login.config", ds.getKerberosJaasConf());
            Configuration conf = new Configuration();
            conf.set("hadoop.security.authentication", "kerberos");
            UserGroupInformation.setConfiguration(conf);
        }
        return jdbcUrlSuffix;
    }

    private String convertColumnType(String type) {
        // KUDU开启INT BIGINT DOUBLE SMALLINT STRING
        if ("INT32".equals(type) || "INT8".equals(type) || "INT".equals(type)) {
            type = "INT";
        } else if ("INT64".equals(type) || "BIGINT".equals(type)) {
            type = "BIGINT";
        } else if ("FLOAT".equals(type) || "DOUBLE".equals(type)) {
            type = "DOUBLE";
        } else if ("SMALLINT".equals(type)) {
            // 不做变更
        } else {
            type = "STRING";
        }
        return type;
    }

    public DmpAgentResult previewData_bak(DmpAgentDatasourceInfo ds, String tableName) {
        Map<String, Object> map = new HashMap<String, Object>();
        String kuduClients = ds.getEndpoint();
        KuduClient client = null;
        KuduSession session = null;
        //List<Map<String, Object>> headerList1 = new ArrayList<Map<String, Object>>();
        List<String> headerList = new ArrayList<String>();
        List<List<Object>> resultList = new ArrayList<List<Object>>();
        try {
            /*if (DmpDsAgentConstants.IS_ENABLED_KERBEROS)
                JDBCUtils.initKerberosENV4KUDU();*/
            client = new KuduClient.KuduClientBuilder(kuduClients).build();
            session = client.newSession();

            KuduTable table = client.openTable(tableName);
            List<ColumnSchema> schema = table.getSchema().getColumns();
            List<String> projectColumns = new ArrayList<String>(1);
            for (ColumnSchema cs : schema) {
                headerList.add(cs.getName().toLowerCase());
                projectColumns.add(cs.getName());
            }
            map.put("header", headerList);

            //ScanRequestPB srp = new ScanRequestPB(false);
            KuduScanner scanner = client.newScannerBuilder(table).setProjectedColumnNames(projectColumns).build();
            while (scanner.hasMoreRows()) {
                RowResultIterator results = scanner.nextRows();
                int i = 0;
                while (results.hasNext()) {
                    RowResult result = results.next();
                    Map<String, Object> m = new HashMap<String, Object>();
                    List<Object> l = new ArrayList<Object>();
                    for (int j = 0; j < projectColumns.size(); j++) {
                        if (result.getColumnType(j) == Type.STRING) {
                            m.put(projectColumns.get(j), result.getString(j));
                            l.add(result.getString(j));
                        } else if (result.getColumnType(j) == Type.BINARY) {
                            m.put(projectColumns.get(j), result.getBinary(j));
                            l.add(result.getBinary(j));
                        } else if (result.getColumnType(j) == Type.BOOL) {
                            m.put(projectColumns.get(j), result.getBoolean(j));
                            l.add(result.getBoolean(j));
                        } else if (result.getColumnType(j) == Type.DOUBLE) {
                            m.put(projectColumns.get(j), result.getDouble(j));
                            l.add(result.getDouble(j));
                        } else if (result.getColumnType(j) == Type.FLOAT) {
                            m.put(projectColumns.get(j), result.getFloat(j));
                            l.add(result.getFloat(j));
                        } else if (result.getColumnType(j) == Type.INT32) {
                            m.put(projectColumns.get(j), result.getInt(j));
                            l.add(result.getInt(j));
                        } else if (result.getColumnType(j) == Type.INT64) {
                            m.put(projectColumns.get(j), result.getLong(j));
                            l.add(result.getLong(j));
                        } else if (result.getColumnType(j) == Type.INT8) {
                            m.put(projectColumns.get(j), result.getShort(j));
                            l.add(result.getShort(j));
                        }
                        resultList.add(l);
                    }
                    if (i == 5) break;
                    i++;
                }
                break;
            }
            map.put("result", resultList);
        } catch (Exception e) {
            e.printStackTrace();
            return new DmpAgentResult(ResultCode.EXCEPTION, "KUDU数据预览异常：" + e.getMessage());
        } finally {
            try {
                if (session != null && !session.isClosed())
                    session.close();
            } catch (KuduException e) {
                e.printStackTrace();
            }
            if (client != null)
                try {
                    client.close();
                } catch (KuduException e) {
                    e.printStackTrace();
                }
        }
        return new DmpAgentResult(ResultCode.SUCCESS, gson.toJson(map));
    }

    public static void main(String[] args) {
        Connection connection = null;
        ResultSet rs = null;
        PreparedStatement ps = null;
        String JDBC_DRIVER = "com.cloudera.impala.jdbc41.Driver";
        // String CONNECT_URL = "jdbc:impala://10.0.53.177:21050/kudu_demo_imp";
        String CONNECT_URL = "jdbc:impala://10.0.53.177:21050/tmp";
        // String sql = "show tables";
        String sql = "CREATE TABLE tmp.test (" +
                "id INT NOT NULL," +
                "member_id BIGINT NULL," +
                "small_id SMALLINT NULL," +
                "double_c DOUBLE NULL," +
                "etl_create_time STRING NULL," +
                "etl_update_time STRING NULL," +
                "PRIMARY KEY (id)" +
                ") PARTITION BY HASH (id) PARTITIONS 2 STORED AS KUDU";
        // String sql = "DESCRIBE hht_impala_1";
        try {
            /*CONNECT_URL += ";AuthMech=1;KrbHostFQDN=jtjzvdra116;KrbServiceName=impala";
            System.setProperty("java.security.krb5.conf", "D:/keytab/krb5.conf");
            System.setProperty("java.security.auth.login.config", "D:/keytab/jaas.conf");
            Configuration conf = new Configuration();
            conf.set("hadoop.security.authentication", "kerberos");
            UserGroupInformation.setConfiguration(conf);*/

            Class.forName(JDBC_DRIVER);
            connection = DriverManager.getConnection(CONNECT_URL);
            ps = connection.prepareStatement(sql);
            ps.execute();
            // rs = ps.executeQuery();
            /*while (rs.next()) {
                System.out.println(rs.getString("name"));
                System.out.print(rs.getString("name").toLowerCase() + " = ");
                System.out.print(rs.getString("type").toUpperCase() + " = ");
                System.out.println(rs.getString("comment") + " = ");
            }*/
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            JDBCUtils.disconnect(connection, rs, ps);
        }
    }

}
