package com.jz.common.exception;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.dao.DataIntegrityViolationException;
import org.springframework.http.HttpStatus;
import org.springframework.http.converter.HttpMessageNotReadableException;
import org.springframework.validation.BindException;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.web.HttpMediaTypeNotSupportedException;
import org.springframework.web.HttpRequestMethodNotSupportedException;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.MissingServletRequestParameterException;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.ResponseStatus;

import com.jz.common.constant.JsonResult;
import com.jz.common.constant.ResultCode;
import io.micrometer.core.instrument.config.validate.ValidationException;

@ControllerAdvice
@ResponseBody
public class GlobalExceptionHandler {

	private static Logger logger = LoggerFactory.getLogger(GlobalExceptionHandler.class);

	/**
	 * 缺少请求参数
	 */
	@ResponseStatus(HttpStatus.OK)
	@ExceptionHandler(MissingServletRequestParameterException.class)
	public JsonResult handleMissingServletRequestParameterException(MissingServletRequestParameterException e) {
		logger.error("缺少请求参数", e);
		return new JsonResult(ResultCode.PARAMS_ERROR, "required_parameter_is_not_present");
	}

	/**
	 * 403 - Bad Request
	 */
	@ResponseStatus(HttpStatus.BAD_REQUEST)
	@ExceptionHandler(HttpMessageNotReadableException.class)
	public JsonResult handleHttpMessageNotReadableException(HttpMessageNotReadableException e) {
		logger.error("参数解析失败", e);
		return new JsonResult(ResultCode.PARAMS_ERROR, "could_not_read_json");
	}

	/**
	 * 参数验证失败
	 */
	@ResponseStatus(HttpStatus.OK)
	@ExceptionHandler(MethodArgumentNotValidException.class)
	public JsonResult handleMethodArgumentNotValidException(MethodArgumentNotValidException e) {
		logger.error("参数验证失败", e);
		BindingResult result = e.getBindingResult();
		FieldError error = result.getFieldError();
		String field = error.getField();
		String code = error.getDefaultMessage();
		String message = String.format("%s:%s", field, code);
		return new JsonResult(ResultCode.PARAMS_ERROR, message);
	}

	/**
	 * 403 - Bad Request
	 */
	@ResponseStatus(HttpStatus.BAD_REQUEST)
	@ExceptionHandler(BindException.class)
	public JsonResult handleBindException(BindException e) {
		logger.error("参数绑定失败", e);
		BindingResult result = e.getBindingResult();
		FieldError error = result.getFieldError();
		String field = error.getField();
		String code = error.getDefaultMessage();
		String message = String.format("%s:%s", field, code);
		//return new AjaxResult().failure(message);
		return new JsonResult(ResultCode.PARAMS_ERROR, message);
	}


	/**
	 * 403 - Bad Request
	 */
	@ResponseStatus(HttpStatus.OK)
	@ExceptionHandler(ValidationException.class)
	public JsonResult handleValidationException(ValidationException e) {
		logger.error("参数验证失败", e);
		return new JsonResult(ResultCode.PARAMS_ERROR, "validation_exception:");
	}

	/**
	 * 405 - Method Not Allowed
	 */
	@ResponseStatus(HttpStatus.METHOD_NOT_ALLOWED)
	@ExceptionHandler(HttpRequestMethodNotSupportedException.class)
	public JsonResult handleHttpRequestMethodNotSupportedException(HttpRequestMethodNotSupportedException e) {
		logger.error("不支持当前请求方式", e);
		return new JsonResult(ResultCode.NOT_SUPPORTED, "request_method_not_supported:");
	}

	/**
	 * 415 - Unsupported Media Type
	 */
	@ResponseStatus(HttpStatus.UNSUPPORTED_MEDIA_TYPE)
	@ExceptionHandler(HttpMediaTypeNotSupportedException.class)
	public JsonResult handleHttpMediaTypeNotSupportedException(Exception e) {
		logger.error("不支持当前媒体类�?", e);
		return new JsonResult(ResultCode.UNSUPPORTED_MEDIA_TYPE, "content_type_not_supported:");
	}

	/**
	 * 500 - Internal Server Error
	 */
	@ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
	@ExceptionHandler(ServiceException.class)
	public JsonResult handleServiceException(ServiceException e) {
		logger.error("业务逻辑异常", e);
		return new JsonResult(ResultCode.INTERNAL_SERVER_ERROR, "业务逻辑异常!");
	}

	/**
	 * 500 - Internal Server Error
	 */
	@ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
	@ExceptionHandler(Exception.class)
	public JsonResult handleException(Exception e) {
		logger.error("通用逻辑异常", e);
		return new JsonResult(ResultCode.INTERNAL_SERVER_ERROR, "通用逻辑异常");
	}

	/**
	 * 操作数据库出现异�?:名称重复，外键关�?
	 */
	@ResponseStatus(HttpStatus.INTERNAL_SERVER_ERROR)
	@ExceptionHandler(DataIntegrityViolationException.class)
	public JsonResult handleException(DataIntegrityViolationException e) {
		logger.error("操作数据库出现异常:", e);
		return new JsonResult(ResultCode.NOT_SUPPORTED, "操作数据库出现异常：字段重复、有外键关联�?");
	}
}
